<?PHP
/*
 *   Copyright (C) 2011 by Rick Schnicker KD0OSS
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


$RCT_NONE = 0;
$RCT_ACK = 1;
$RCT_NAK = 2;
$RCT_RANDOM = 3;
$RCT_CALLSIGNS = 4;
$RCT_REPEATER = 5;
$RCT_STARNET = 6;

$RCFS_NORMAL = 0;
$RCFS_LOGIN = 1;
$RCFS_HASH = 2;
$RCFS_GET_REPEATERS = 3;
$RCFS_GET_STARNETS = 4;
$RCFS_GET_HEADER = 5;

$BUFFER_LENGTH = 2000;
$MAX_RETRIES = 3;

$m_inLength = 0;
$m_outLength = 0;
$m_retryCount = 0;
$m_inBuffer;
$m_outBuffer;
$m_type = 0;
$m_loggedIn = false;
$m_state = 0;


function readType($hd)
{
    global $BUFFER_LENGTH;
    global $m_inBuffer;
    global $m_retryCount;
    global $m_inLength;
    global $m_type;
    global $m_address;
    global $RCT_NONE;
    global $RCT_ACK;
    global $RCT_NAK;
    global $RCT_RANDOM;
    global $RCT_CALLSIGNS;
    global $RCT_REPEATER;
    global $RCT_STARNET;
    
    $m_type = $RCT_NONE;

    $m_inBuffer = stream_socket_recvfrom($hd, $BUFFER_LENGTH, 0, $address);
  //  echo substr($m_inBuffer,0,3)."\n";
    
    if ($address != "") $m_address = $address;
  //  echo $address."\n";
    if ($m_inBuffer == "")
        return $m_type;

    $m_inLength = count($m_inBuffer);

    if (strcmp(substr($m_inBuffer,0,3), "ACK") == 0)
    {
        $m_retryCount = 0;
        $m_type = $RCT_ACK;
        return $m_type;
    }
    else if (strcmp(substr($m_inBuffer,0,3), "NAK") == 0)
    {
        $m_retryCount = 0;
        $m_type = $RCT_NAK;
        return $m_type;
    }
    else if (strcmp(substr($m_inBuffer,0,3), "RND") == 0)
    {
        $m_retryCount = 0;
        $m_type = $RCT_RANDOM;
        return $m_type;
    }
    else if (strcmp(substr($m_inBuffer,0,3), "CAL") == 0)
    {
        $m_retryCount = 0;
        $m_type = $RCT_CALLSIGNS;
        return $m_type;
    }
    else if (strcmp(substr($m_inBuffer,0,3), "RPT") == 0)
    {
        $m_retryCount = 0;
        $m_type = $RCT_REPEATER;
        return $m_type;
    }
    else if (strcmp(substr($m_inBuffer,0,3), "SNT") == 0)
    {
        $m_retryCount = 0;
        $m_type = $RCT_STARNET;
        return $m_type;
    }

    return $m_type;
} /** end readType **/


function readNAK()
{
    global $m_inBuffer;
    global $RCT_NAK;
    global $m_type;
    
    if ($m_type != $RCT_NAK)
        return "";

    $text = substr($m_inBuffer, 0, 3);

    return $text;
} /** end readNAK **/


function readRandom()
{
    global $m_inBuffer;
    global $m_type;
    global $RCT_RANDOM;
    
    if ($m_type != $RCT_RANDOM)
        return 0;

    $random = substr($m_inBuffer, 3, 4);
    
    return $random;
} /** end readRandom **/


function login($hd)
{
    global $m_loggedIn;
    global $m_address;
    global $m_retryCount;
    global $m_outLength;
    global $m_outBuffer;
    
    if ($m_loggedIn)
        return false;

    if ($m_address == "")
        return false;

    $m_outBuffer = "LIN";
    $m_outLength = 3;

    $ret = fwrite($hd, $m_outBuffer, $m_outLength);
    if (!$ret)
    {
        $m_retryCount = 0;
        return false;
    }
    else
    {
        $m_retryCount = 1;
        return true;
    }
} /** end login **/


function setLoggedIn($set)
{
    global $m_loggedIn;
    
    $m_loggedIn = $set;
} /** end setLoggedIn **/


function sendHash($hd, $rnd)
{
    global $m_retryCount;
    global $m_loggedIn;
    global $m_password;
    global $m_address;
    
    $len = strlen($m_password);
    $in = $rnd;
    for ($i = 0; $i < strlen($m_password); $i++)
        $in[$i+4] = $m_password[$i];

    $hash = hash("sha256", $in, true);

    assert($hash != NULL);
    assert($len > 0);

    if ($m_loggedIn || $m_retryCount > 0)
        return false;

    $m_outBuffer = "SHA";

    for ($i=0;$i<strlen($hash);$i++)
        $m_outBuffer[$i+3] =  $hash[$i];

    $ret = stream_socket_sendto($hd, $m_outBuffer, 0, $m_address);
    
    if ($ret)
    {
        $m_retryCount = 0;
        return false;
    }
    else
    {
        $m_retryCount = 1;
        return true;
    }
} /** end sendHash **/


function logout($hd)
{
    global $m_retryCount;
    global $m_loggedIn;
    global $m_outBuffer;
    global $m_address;
    
    if (!$m_loggedIn || $m_retryCount > 0)
        return false;

    $m_outBuffer = "LOG";

    for ($i = 0; $i < 5; $i++)
    {
        $ret = stream_socket_sendto($hd, $m_outBuffer, 0, $m_address);
        if ($ret)
        {
            $m_retryCount = 0;
            return false;
        }
    }

    $m_retryCount = 1;

    return true;
} /** end logout **/


function retry($hd)
{
    global $MAX_RETRIES;
    global $m_retryCount;
    global $m_outBuffer;
    global $m_address;
    
    if ($m_retryCount > 0)
    {
        $m_retryCount++;
        if ($m_retryCount >= $MAX_RETRIES)
        {
            $m_retryCount = 0;
            return false;
        }
        stream_socket_sendto($hd, $m_outBuffer, 0, $m_address);
    }

    return true;
} /** end retry **/


function getCallsigns($hd)
{
    global $m_retryCount;
    global $m_outBuffer;
    global $m_address;
    global $m_loggedIn;
    
    if (!$m_loggedIn || $m_retryCount > 0)
        return false;

    $m_outBuffer = "GCS";

    $ret = stream_socket_sendto($hd, $m_outBuffer, 0, $m_address);
    if (!$ret)
    {
        $m_retryCount = 0;
        return false;
    }
    else
    {
        $m_retryCount = 1;
        return true;
    }
} /** end getCallsigns **/


function getRepeater($hd, $callsign)
{
    global $m_retryCount;
    global $m_outBuffer;
    global $m_address;
    global $m_loggedIn;
    
    if ($callsign == "") return;

    if (!$m_loggedIn || $m_retryCount > 0)
        return false;

    $m_outBuffer =  "GRP        ";

    for ($i = 0; $i < strlen($callsign); $i++)
        $m_outBuffer[$i+3] = $callsign[$i];

    $ret = stream_socket_sendto($hd, $m_outBuffer, 0, $m_address);
    if (!$ret)
    {
        $m_retryCount = 0;
        return false;
    }
    else
    {
        $m_retryCount = 1;
        return true;
    }
} /** end getRepeater **/


function getStarNet($hd, $callsign)
{
    global $m_retryCount;
    global $m_outBuffer;
    global $m_address;
    global $m_loggedIn;
    
    if ($callsign == "") return;

    if (!$m_loggedIn || $m_retryCount > 0)
        return false;

    $m_outBuffer =  "GSN        ";

    for ($i = 0; $i < strlen($callsign); $i++)
        $m_outBuffer[$i+3] = $callsign[$i];

    $ret = stream_socket_sendto($hd, $m_outBuffer, 0, $m_address);
    if (!$ret)
    {
        $m_retryCount = 0;
        return false;
    }
    else
    {
        $m_retryCount = 1;
        return true;
    }
} /** end getStarNet **/


function logoffStarNet($hd, $callsign, $user)
{
    global $m_retryCount;
    global $m_outBuffer;
    global $m_address;
    global $m_loggedIn;
    
    if ($callsign == "") return;
    if ($user == "") return;

    if (!$m_loggedIn || $m_retryCount > 0)
        return false;

    $m_outBuffer =  "LGO        ";

    for ($i = 0; $i < strlen($callsign); $i++)
        $m_outBuffer[$i+3] = $callsign[$i];

    $m_outBuffer .= "        ";
    
    for ($i = 0; $i < strlen($user); $i++)
        $m_outBuffer[$i+3] = $user[$i];

    $ret = stream_socket_sendto($hd, $m_outBuffer, 0, $m_address);
    if (!$ret)
    {
        $m_retryCount = 0;
        return false;
    }
    else
    {
        $m_retryCount = 1;
        return true;
    }
} /** end logoffStarNet **/


function linkto($hd, $callsign, $reconnect, $reflector)
{
    global $m_retryCount;
    global $m_outBuffer;
    global $m_address;
    global $m_loggedIn;
    
    if ($callsign == "") return;

    if (!$m_loggedIn || $m_retryCount > 0)
        return false;

    $m_outBuffer =  "LNK        ";

    for ($i = 0; $i < strlen($callsign); $i++)
        $m_outBuffer[$i+3] = $callsign[$i];

    $m_outBuffer[14] = chr($reconnect & 0xff000000);
    $m_outBuffer[13] = chr($reconnect & 0x00ff0000);
    $m_outBuffer[12] = chr($reconnect & 0x0000ff00);
    $m_outBuffer[11] = chr($reconnect & 0x000000ff);

    $m_outBuffer .= "        ";
    
    for ($i = 0; $i < strlen($reflector); $i++)
        $m_outBuffer[$i+15] = $reflector[$i];

    $ret = stream_socket_sendto($hd, $m_outBuffer, 0, $m_address);
    if (!$ret)
    {
        $m_retryCount = 0;
        return false;
    }
    else
    {
        $m_retryCount = 1;
        return true;
    }
} /** end linkto **/


function sendRemoteCommand($mod, $command, $password)
{
    global $m_address;
    global $m_port;
    global $m_password;
    global $m_inBuffer;
    global $m_state;
    global $RCT_NONE;
    global $RCT_ACK;
    global $RCT_NAK;
    global $RCT_RANDOM;
    global $RCT_CALLSIGNS;
    global $RCT_REPEATER;
    global $RCT_STARNET;
    global $RCFS_NORMAL;
    global $RCFS_LOGIN;
    global $RCFS_HASH;
    global $RCFS_GET_REPEATERS;
    global $RCFS_GET_STARNETS;
    global $RCFS_GET_HEADER;
    
 
    $hd = stream_socket_client("udp://$m_address:$m_port", $errno, $errstr);
    {
        $m_password = $password;
        if (login($hd))
        {
            $m_state = $RCFS_LOGIN;
        }
        
        $c = 1;
        while ($c == 1)
        {
            $type = readType($hd);
            switch ($type)
            {
                case $RCT_NONE:
                    retry($hd);
                break;

                case $RCT_NAK:
                {
                    if ($m_state == $RCFS_HASH || $m_state == $RCFS_LOGIN)
                        setLoggedIn(false);

                    $text = readNAK();
                    $m_state = $RCFS_NORMAL;
                    return $text;
                }
                break;

                case $RCT_RANDOM:
                {
                    $rnd = readRandom();
                    sendHash($hd, $rnd);
                    $m_state = $RCFS_HASH;
                }
                break;

                case $RCT_ACK:
                {
                    setLoggedIn(true);
                    $m_state = $RCFS_HASH;
                    $fields = explode(",", $command);
                    if ($fields[0] == "link")
                        linkto($hd, $mod, 0, $fields[1]);  // Link to given reflector.  Blank mod unlinks
                    if ($fields[0] == "repeater")
                        getRepeater($hd, $fields[1]);      // get link data for given repeater callsign
                    if ($fields[0] == "starnet")
                        getStarNet($hd, $fields[1]);       // get starnet group info
                    logout($hd);
                    $c = 0;
                }
                break;
                
                case $RCT_CALLSIGNS:
                {
                    return substr($m_inBuffer, 3);
                    logout($hd);
                    $c = 0;
                }
                break;
                        
                case $RCT_REPEATER:
                {
                    return substr($m_inBuffer, 3);
                    logout($hd);
                    $c = 0;
                }
                break;
                        
                case $RCT_STARNET:
                {
                    return substr($m_inBuffer, 3);
                    logout($hd);
                    $c = 0;
                }
                break;
            }
        }
        fclose($hd);
    }
    return "ok";
} /** end sendRemoteCommand **/
    
?>
